import sys
sys.path.append("..")
sys.path.append(".")
sys.path.append("../keysql_client/")
sys.path.append("../..")
#import keysql_client
from keysql_client.keysql_connection import KeySqlConnection
from keysql_client.keysql_version_statement import KeySqlVersionStatement
from keysql_client._version import get_keysql_server_version, get_python_driver_version

class KeySqlClient:

    def __init__(self, host: str, port: int):
        self._host = host
        self._port = port
        self._connection = None


    def connect(self, username: str, password: str, schema="", secure_ssl=True):
        self._connection = KeySqlConnection(self._host,self._port,username,password, secure_ssl, schema)
        return self._connection

    def client_version(self):
        return get_python_driver_version()


    def server_version(self):
        return get_keysql_server_version()


    def check_version(self):
        """
        :return: True if the server build version corresponds to the version written in _version file
        """
        # output
        # [{'version_info': {'product_name': 'KeySQL Server', 'version_num': '2.12.0',
        # 'release_date': '2022-02-13 10:36:50', 'build_num': '1837 (KEYARK-492:a236405)',
        # 'default_dbms': 'Provider: PostgreSQL, SQL dialect: PostgreSQL'}}]

        version_stm = KeySqlVersionStatement()
        reply = self._connection.execute(KeySqlVersionStatement(), format='json')

        version = list(map(int, reply[0]['version_info']['version_num'].strip().split('.')))

        python_server_version = get_keysql_server_version()
        python_client_version = get_python_driver_version().strip().split('.')
        python_client_version[0] = python_client_version[0][1:]
        python_client_version = list(map(int, python_client_version))

        return len(version) == 3 and \
               len(python_server_version) == 3 and \
               python_server_version[0] == version[0] and \
               python_server_version[1] == version[1]
