#/**********************************************************************************
#* Copyright (C) 2017-2020 by KEYARK, INC. - All Rights Reserved.
#* All information contained herein is proprietary and confidential to KEYARK, INC.
#* Any use, reproduction, or disclosure without the written
#* permission of KEYARK, INC. is prohibited.
#* Written by Natalia Belyaeva <nbelyaeva@keyark.com>
#***********************************************************************************/

#from unit_tests_keysql_driver.test_keysql_base import *
from keysql_client.key_object import KeyObjectNumber, KeyObjectChar, KeyObjectDate, KeyObjectComposition, KeyObjectMultiple
from keysql_client.keysql_catalog_statement import *
from keysql_client.keysql_store_statement import *
from keysql_client.key_object_statement import *
from keysql_client.keysql_parser import *
from keysql_client.keysql_connector import KeySqlConnector
from keysql_client.keysql_connection import KeySqlConnection
import logging

class KeySqlBuilder:
    def __init__(self, my_port, my_connector_host, my_user_name):
        self._my_port = my_port
        self._my_user_name = my_user_name
        self._my_connector_host = my_connector_host
        self._my_connection = KeySqlConnection(KeySqlConnector(my_connector_host, my_port), my_user_name)


    def create_catalog(self, catalog):
        reply = self._my_connection.execute(KeySqlCreateCatalogStatement(catalog))
        header = self._my_connection.get_header_reply()
        if header["exit_code"] != 0:
            logging.exception('create_catalog: ', header["exit_code"], reply)
            raise KeySqlException()


    def create_store(self, store, catalog):
        reply = self._my_connection.execute(KeySqlCreateStoreStatement(store, catalog).create_request())
        header = self._my_connection.get_header_reply()
        if header["exit_code"] != 0:
            logging.exception('create_store: ', header["exit_code"],  reply)
            raise KeySqlException()


    def create_stores(self, store_list, catalog):
        for store in store_list:
            reply = self._my_connection.execute(KeySqlCreateStoreStatement(store, catalog).create_request())
            header = self._my_connection.get_header_reply()
            if header["exit_code"] != 0:
                logging.exception('create_stores: ', header["exit_code"], reply)
                raise KeySqlException()


    def create_keyobjects(self, key_object_list, catalog):
        for key_object_item in key_object_list:
            reply = self._my_connection.execute(CreateKeyObjectStatement(key_object_item, catalog).create_request())
            header = self._my_connection.get_header_reply()
            if header["exit_code"] != 0:
                logging.exception('create_keyobjects: ', header["exit_code"], reply)
                raise KeySqlException()

    # type(classname, superclasses, attributedict)
    @staticmethod
    def keyqsl_object_to_instance(key_obj):
        if type(key_obj) == KeyObjectChar:
            tmp = KeySql_Instance(key_obj.__name__)
        elif type(key_obj) == KeyObjectNumber:
            tmp = KeySql_Instance(key_obj.__name__)
        elif  type(key_obj) == KeyObjectDate:
            tmp = KeySql_Instance(key_obj.__name__)
        elif type(key_obj) == KeyObjectMultiple:
            tmp = KeySql_Instance(key_obj.__name__, [])
        elif type(key_obj) == KeyObjectComposition:
            tmp = KeySql_Instance(key_obj.__name__)
            dic_val = {}
            for item in key_obj.__children__:
                item_dict = KeySqlBuilder.keyqsl_object_to_instance(item)
                dic_val[item_dict._name] = item_dict
            tmp._value = dic_val

        return tmp


    def insert_values_into_keysql_class(self, inp_line, keyobjects_dict):
        return


if __name__ == "__main__":
    pass
