#/**********************************************************************************
#* Copyright (C) 2017-2020 by KEYARK, INC. - All Rights Reserved.
#* All information contained herein is proprietary and confidential to KEYARK, INC.
#* Any use, reproduction, or disclosure without the written
#* permission of KEYARK, INC. is prohibited.
#* Written by Natalia Belyaeva <nbelyaeva@keyark.com>
#***********************************************************************************/

from keysql_client.keysql_statement import KeySqlStatement
from keysql_client.keysql_sequence_statement import KeySqlDropSequenceStatement, KeySqlDropSequenceIfExistsStatement
from keysql_client.key_object import *
from keysql_client.keysql_exception import KeySqlExceptionMessage, KeySqlException
from keysql_client.keysql_client_exception import KeySqlClientException
from keysql_client.keysql_parser import KeySqlParser
import json
#from keysql_client.keysql_mapper_json import KeySqlMapperJson

class KeySqlCreateCatalogStatement(KeySqlStatement):
    # members
    _name = None
    def __init__(self, name : str = None):
        self._name = name

    def create_request(self) -> str:
        return 'create catalog {}'.format(self._name)

    #def process_reply(self, reply: str, header = None) -> None:
    #    # nothing to process - input is ''
    #    return reply


class KeySqlListCatalogsStatement(KeySqlStatement):
    # members

    def __init__(self, schema=None, all=False):
        """
        If schema is None, use default schema
        :param schema: if schema is not None
        :param all: if True list all catalogs (full name): show catalogs ALL
                    if False and schema is given show catalogs from schema: show catalogs from [schema] <shcema_name>
        """
        self.schema = schema
        self.all = all

    def create_request(self) -> str:
        if self.all:
            # return 'show catalogs *'
            return 'list catalogs ALL'
        if self.schema:
            # return 'show catalogs {}.*'.format(self.schema)
            # "list catalogs from [schema] <schema_name>"
            return 'list catalogs from schema {}'.format(
                self.schema)  # return 'list catalogs {}.*'.format(self.schema)  #

        if self.schema is None:
            # return 'show catalogs'
            return 'list catalogs'

    def process_reply(self, reply, header=None) -> list:
        if reply == "":
            return []

        if header and header['format'] == 'json':
            if type(reply) is str:
                all_items = json.loads(reply)
            else:
                all_items = reply
            # all_items_names = [rec["CATALOG_NAME"] for rec in all_items]
            # return all_items_names
            return all_items

        elif header and header['format'] == 'keysql':
            all_items_names = [rec.instance_value() for rec in reply]
            # print(reply)
            # print(reply[0].instance_name())
            # print(reply[0].instance_value())
            # all_items_names = [rec.instance_value()["CATALOG_NAME"] for rec in reply]
            return all_items_names
        else:
            raise KeySqlException(KeySqlExceptionMessage.FORMAT_KEYSQL_JSON)

        # remove first { and last}  Catalog <name>{...} and return what is between {}
        # ind = all_items_names.find('{')
        # all_items_names = all_items_names[ind + 1 :-1].split(',\n')
        # if all_items_names[0].upper().startswith('CATALOG'):
        #    all_items_names = all_items_names[1:]
        # return all_items_names.split(',\n')
        # return all_items_names


class KeySqlRenameCatalogStatement(KeySqlStatement):
    _old_name = None
    _new_name = None

    def __init__(self, old_name: str = None, new_name: str = None, schema=None):
        self._old_name = old_name
        self._new_name = new_name
        self._schema = schema

    def create_request(self) -> str:
        if self._schema is None:
            return 'rename catalog {} to {} '.format(self._old_name, self._new_name)
        return 'rename catalog {} to {} in schema {} '.format(self._old_name, self._new_name, self._schema)

    def process_reply(self, reply: str, header=None) -> None:
        pass


class KeySqlAlterCatalogStatement(KeySqlStatement):
    _old_name = None
    _new_name = None

    def __init__(self, old_name: str = None, new_name: str = None):
        self._old_name = old_name
        self._new_name = new_name

    def create_request(self) -> str:
        return 'alter catalog {} with name = {} '.format(self._old_name, self._new_name)

    def process_reply(self, reply: str, header=None) -> None:
        pass


class KeySqlDescribeCatalogStatement(KeySqlStatement):
    def __init__(self, name: str = None):
        self._name = name

    def create_request(self) -> str:
        return "describe catalog {}".format(self._name)

    def process_reply(self, reply: str, header=None) -> None:
        pass

class KeySqlDescribeCatalogsStatement(KeySqlStatement):

    def __init__(self, schema_name: str = None):
        self.schema_name = schema_name

    def create_request(self) -> str:
        return "describe catalogs from schema {}".format(self.schema_name)

    #def process_reply(self, reply: str, header=None) -> None:
    #    pass



class KeySqlDropCatalogStatement(KeySqlStatement):
    # members
    _name = None

    def __init__(self, name: str = None) -> None:
        self._name = name

    def create_request(self) -> str:
        return 'drop catalog {}'.format(self._name)

    #def process_reply(self, reply: str, header=None) -> None:
    #    return header['instance_count'] if header else reply

class KeySqlDropCatalogIfExistsStatement(KeySqlStatement):
    # members
    _name = None

    def __init__(self, name: str = None) -> None:
        self._name = name

    def create_request(self) -> str:
        return 'drop catalog  if exists {}'.format(self._name)


