
from keysql_client.keysql_connection_async_coroutine import KeySqlConnectionAsyncCoroutine

from tornado.ioloop import IOLoop
from tornado.gen import coroutine

class KeySqlClientAsyncCoroutine:
    def __init__(self, host: str, port: int, ioloop=IOLoop.current()):  # 'async', 'async_coroutine'
        """
        KeySqlClient for asyncronious running
        :param host: host - string
        :param port: port - integer
        :param ioloop:  IOLoop.current() - default
        :param conn_type:  'async' - default (for native python, 'async_coroutine - for tornado coroutine
        """
        self._host = host
        self._port = port
        self._ioloop = ioloop
        self._connection = KeySqlConnectionAsyncCoroutine(self._host, self._port, self._ioloop)

    @coroutine
    def connect(self, username: str, password: str, ssl=True):
        #self._connection = KeySqlConnectionAsyncCoroutine(self._host, self._port, self._ioloop)
        yield self.connect_base(username, password, ssl=ssl)
        return self._connection

    @coroutine
    def connect_base(self, username: str, password: str, ssl=True):
        """
        Connect with server using username and password
        :param username:  user name, string
        :param password:  password, string
        :param ssl: default: true (secure connection), False - non-secure connection
        :return:
        """
        yield self._connection.connect_user(username, password, ssl=ssl)

