#/**********************************************************************************
#* Copyright (C) 2017-2020 by KEYARK, INC. - All Rights Reserved.
#* All information contained herein is proprietary and confidential to KEYARK, INC.
#* Any use, reproduction, or disclosure without the written
#* permission of KEYARK, INC. is prohibited.
#* Written by Natalia Belyaeva <nbelyaeva@keyark.com>
#***********************************************************************************/

import ssl
import tempfile
import os
import logging
import time
from tornado import concurrent
from tornado.ioloop import IOLoop           #  Facebook 2009  - created ioloop
from tornado.tcpclient import TCPClient
from tornado.gen import convert_yielded

import locale
locale.setlocale(locale.LC_ALL, 'en_US.utf-8')

import tracemalloc

# call: StatementType(4)
from keysql_client.keysql_exception import KeySqlException, KeySqlConnectException, KeySqlExceptionMessage
from keysql_client.keysql_message_header import MessageHeader

ssl.get_default_verify_paths()

# C:\Users\natal\AppData\Local\JetBrains\PyCharmCE2020.2\python_stubs\328341865\   #    _ssl.py

class KeySqlConnectorAsync:
    pem = ["-----BEGIN CERTIFICATE-----\n",
              "MIICKzCCAZQCCQDFtaRb2MBPLTANBgkqhkiG9w0BAQsFADBaMQswCQYDVQQGEwJV\n",
              "UzELMAkGA1UECAwCQ0ExETAPBgNVBAcMCE1pbHBpdGFzMQ0wCwYDVQQKDAR0ZXN0\n",
              "MQ0wCwYDVQQLDAR0ZXN0MQ0wCwYDVQQDDAR0ZXN0MB4XDTE5MDkxMDA0NDI1MVoX\n"
              "DTI5MDkwNzA0NDI1MVowWjELMAkGA1UEBhMCVVMxCzAJBgNVBAgMAkNBMREwDwYD\n"
              "VQQHDAhNaWxwaXRhczENMAsGA1UECgwEdGVzdDENMAsGA1UECwwEdGVzdDENMAsG\n"
              "A1UEAwwEdGVzdDCBnzANBgkqhkiG9w0BAQEFAAOBjQAwgYkCgYEA013a3IUOihH9\n"
              "g5BvTrVtkHsq3utFgSXfQzblaN90BjLHmEisdbaldbPTT6TXvKn3v9C7J9hnRa2h\n",
              "fGCArVvyUMF6qNcCezre4ESskD0Z7Blec2Dc0uR70QCDFIoOyYjeja7p4sAumYYe\n",
              "UYt7evCNFLq2kPse/YfbQC9Jpa0oAykCAwEAATANBgkqhkiG9w0BAQsFAAOBgQA6\n",
              "FlMV6JUtGnKnVFNN+jCjILu45kSYThMMntNCqtpcJO99U/2Ng1qQdss+TL4vfx/l\n",
              "y1aNxxydCiD2XZKkjsw1RR924u8sbqzwpnlYmaWBb3JKJ5VuhagjpwgNrdpZr/Sx\n",
              "FpdNklc47mX3SpCgYcBDnssGCKjUs83GBcQYRV+oZg==\n",
              "-----END CERTIFICATE-----\n"]

    def __init__(self, host: str, port:int, ioLoop:IOLoop, timeout:int = None):

        self._host = host
        self._port = port
        self._ioLoop = ioLoop
        self._server_address = (self._host, self._port)

        self._timeout = timeout
        self._connected = False
        self._msg_id = 0
        self._stream = None


    def is_connected(self):
        """
        Check if there is connection with server
        :return: None
        """

        return self._connected

    """
            try:
                connect_msg = bytes("connect " + user + " " + password + "\n",encoding="utf-8")
                if self._stream is None:
                    self._stream = yield TCPClient().connect(self._host, self._port)
                yield self._stream.write(connect_msg)
                reply = yield self._stream.read_until(b"\n")
                reply = reply.decode("ascii")
    """

    async def connect_user(self, user, password):
        reply = None
        try:
            connect_msg = bytes("connect " + user + " " + password + "\n",encoding="utf-8")
            if self._stream is None:
                self._stream = await TCPClient().connect(self._host, self._port)
            self._fd = self._stream.fileno()
            await self._stream.write(connect_msg)
            reply = await self._stream.read_until(b"\n")
            #reply = self._stream.read_until(b"\n")
            reply = reply.decode("ascii")
        except ValueError as ex:   #   Exception as ex:
            future = concurrent.Future()
            future.set_exception(KeySqlConnectException(ex))  # .message)
            return future

        if reply != "OK\n":
            future = concurrent.Future()
            future.set_exception(KeySqlConnectException(reply))
            return future
        self._connected = True
        return True

    # how todo:
    # chunk = self._sock.recv(8192)                             # blocking code
    #chunk = await self._stream.read_bytes(8192, partial=True)  # non-blocking code
    # self._sock.sendall(message.encode())                      #  blocking code
    # await self._stream.write(message.encode())                # non-blocking code

    async def __API_call(self,command: str,expected: str):
        reply = None
        try:
            #self._sock.sendall(bytes(command + "\n",encoding="utf-8"))        #  blocking code
            await self._stream.write(bytes(command + "\n",encoding="utf-8"))  # non-blocking code

            #reply = self._sock.recv(128)                             #  blocking code
            reply = await self._stream.read_bytes(128, partial=True)  # non-blocking code

            pos = reply.find(b'\n')
            reply = reply[0:pos].decode("ascii")
        except ValueError as ex:   #   Exception as ex:
            raise KeySqlConnectException(ex)  # .message)

        if reply == expected:
            return True
        raise KeySqlConnectException(reply)  # .message)

    def disconnect_user(self):
        return self.__API_call("disconnect","OK")


    def abort_query(self):
        return self.__API_call("abort","OK")

    def ping(self):
        return self.__API_call("ping","PONG")


    async def ssl_connect_async(self, user: str = None, password: str = None): # , nonblocking=True):
        """
        SSL connection with server
        :param user: user name
        :param password:  user password
        :return: None
        """
        if self._connected:
            raise KeySqlConnectException(KeySqlExceptionMessage.DISCONNECT_BEFORE_CONNECT)

        if not self._connected:    # _sock_connected:
            try:
                context = ssl.SSLContext(ssl.PROTOCOL_TLSv1_2)
                fd, tmp_path = tempfile.mkstemp()
                with open(tmp_path, 'w') as f:
                    for s in self.pem:
                        f.write(s)
                os.close(fd)
                context.load_verify_locations(tmp_path)

                self._stream = await TCPClient().connect(self._host, self._port,ssl_options=context)

                self._sock_connected = True

            except ValueError as ex:
                raise KeySqlConnectException(ex)

        if user and password:
            cr = await self.connect_user(user,password)
            return cr
        return True


    # TODO: Memo
    #  Async vs not async:
    # self._sock.sendall(message.encode())
    #  async:
    # self._stream = await TCPClient().connect(self._host, self._port)
    # Tornado:
    # BaseIOStream.write(data: Union[bytes, memoryview]) → Future[None]
    # BaseIOStream.read_bytes(num_bytes: int, partial: bool = False) → Awaitable[bytes]


    async def send_resv(self, msg, on_error=None, format='keysql'):
        """
        Send a query to the server
        :param msg: str
        :return: None
        """

        def utf8len(s):
            return len(s.encode('utf-8'))

        if format not in ['keysql', 'json']:
            raise KeySqlException(-1, KeySqlExceptionMessage.FORMAT_KEYSQL_JSON)

        self._msg_id += 1
        header = MessageHeader(self._msg_id, format)
        message = "submit " + header.out_msg(utf8len(msg))
        message += msg

        #self._sock.sendall(message.encode())         #  locking code
        await self._stream.write(message.encode())    # non-blocking code

        reply = ""
        reply_header = None
        header_str = ""
        page = b""

        def get_header(chunk,header_str):
            pos = chunk.find(b'\n')
            if pos > 0:
                header_str += chunk[0:pos].decode("ascii")
                reply_header = header.unpack(header_str)
                return chunk[pos + 1:], "", reply_header

            header_str += chunk.decode("ascii")
            return b"", header_str, None

        while True:
            #chunk = self._sock.recv(8192) # blocking code
            chunk = await self._stream.read_bytes(8192, partial=True) # non-blocking code


            if reply_header is None:
                chunk, header_str, reply_header = get_header(chunk, header_str)
            page += chunk
            if len(page) >= reply_header["message_length"]:
                reply += page[0:reply_header["message_length"]].decode("utf-8")
                if reply_header["page"] == 0:
                    break
                page, header_str, reply_header = get_header(page[reply_header["message_length"] + 1:],header_str)

        if reply_header['exit_code'] != 0:
            logging.error('exit_code: ' + str(reply_header['exit_code']) + ', reply: ' + reply)
            reply_header["err_message"] = reply
            raise KeySqlException(reply_header['exit_code'], reply)

        return reply_header, reply


    def nb_cleanup(self):
        self._ioLoop.remove_handler(self._sock.fileno())


    def _disconnect(self):
        """
        Disconnect with server
        :return: None
        """
        time.sleep(0.5)     # 5 milliseconds
        #self._sock.close()   # blocking
        self._stream.close()  # non-blocking
        self._connected = False


def check_connected(fut):
    print('connected', fut.result())

async def conn1_async(my_local_host, my_port, my_user, my_pass, ioLoop):
    time_start = time.time()
    connector = KeySqlConnectorAsync(my_local_host, my_port, ioLoop)
    await connector.ssl_connect_async(my_user, my_pass)
    #print('connector: ', connector.is_connected())
    header, reply = None, None
    header, reply = await connector.send_resv('select * from world_bank_store')
    if reply:
        print("conn1_async: ", reply[:38])
    for ii in range(30):
        header, reply = None, None
        header, reply = await connector.send_resv('select * from world_bank_store')   # ('show catalogs')
        if reply:
            print("conn1_async: ", ii, '-'*50)
            print(reply[:4], ' ...')
    print("1st finished in {} milisec. 1".format(1000 * (time.time() - time_start)))

    await connector.disconnect_user()
    return "1st finished in {} milisec. 2".format(1000 * (time.time() - time_start))


async def conn2_async(my_local_host, my_port, my_user, my_pass, ioLoop):
    time_start = time.time()
    connector2 = KeySqlConnectorAsync(my_local_host, my_port, ioLoop)
    await connector2.ssl_connect_async(my_user, my_pass)
    #print('connector2:', connector2.is_connected())

    for ii2 in range(50):
        header, reply = None, None
        header, reply = await connector2.send_resv('select * from world_bank_store')
        if reply:
            print("conn2_async: ", ii2, '='*75)
            print(reply[:38], ' ...')
    print("2nd finished in {} milisec. 1".format(1000 * (time.time() - time_start)))

    await connector2.disconnect_user()

    return "2nd finished in {} milisec. 2".format(1000 * (time.time() - time_start))

def callback(f):
    print(f.result())


def two_conn(my_local_host, my_port, my_user, my_pass, ioLoop):
    print('time start: ', time.time())
    f1 = convert_yielded(conn1_async(my_local_host, my_port, my_user, my_pass, ioLoop))
    ioLoop.add_future(f1, callback)
    f2 = convert_yielded(conn2_async(my_local_host, my_port, my_user, my_pass, ioLoop))
    ioLoop.add_future(f2, callback)
    print('time end  : ', time.time())


if __name__ == "__main__":
    pass

    # for testing:
    """
    tracemalloc.start()

    my_local_host = 'my_host'        # put real ip 
    my_port = 6666   # SSL port
    #my_port = 5556

    my_user = "test3"
    my_pass = "test"

    time1 = time.time()
    print('\n both.....')

    ioLoop = IOLoop.current()
    two_conn(my_local_host, my_port, my_user, my_pass, ioLoop)
    ioLoop.start()

    print(time.time())
    """