#/**********************************************************************************
#* Copyright (C) 2017-2020 by KEYARK, INC. - All Rights Reserved.
#* All information contained herein is proprietary and confidential to KEYARK, INC.
#* Any use, reproduction, or disclosure without the written
#* permission of KEYARK, INC. is prohibited.
#* Written by Natalia Belyaeva <nbelyaeva@keyark.com>
#***********************************************************************************/

import socket
import ssl
import tempfile
import sys, os
sys.path.append("..")
sys.path.append(".")
#sys.path.append("../../pyKeySql/keysql_client")
#sys.path.append("../../keysql_client")
sys.path.append("../keysql_client")
import logging
import time
from datetime import datetime
#now = datetime.now().time()    to get time
import threading

import locale
locale.setlocale(locale.LC_ALL, 'en_US.utf-8')
"""
import keysql_client
from keysql_client.keysql_message_header import MessageHeader
from keysql_client.keysql_exception import KeySqlException, KeySqlConnectException, KeySqlExceptionMessage
from keysql_client.keysql_connector_base import KeySqlConnectorBase
"""

#import PyKeySql.keysql_message_header.MessageHeader
#import  keysql_message_header.MessageHeader

#import keysql_message_header

from keysql_client.keysql_message_header import MessageHeader
from keysql_client.keysql_exception import KeySqlException, KeySqlConnectException, KeySqlExceptionMessage
from keysql_client.keysql_connector_base import KeySqlConnectorBase

#from keysql_message_header import MessageHeader

class KeySqlConnector(KeySqlConnectorBase):
    def cut_message(self, mess):
        TOTAL_LEN_BUG_REPORT = 320
        if len(mess) > TOTAL_LEN_BUG_REPORT:
            short_mess = mess[:TOTAL_LEN_BUG_REPORT] + ' ... ' + mess[-TOTAL_LEN_BUG_REPORT:] + '__len(mess): {} smbs'.format(len(mess))
        else:
            short_mess = mess
        return short_mess

    def __init__(self, host: str, port:int, schema: str = "", timeout:int = None):
        super(KeySqlConnector, self).__init__(host, port, schema)
        try:
            self._host = socket.gethostbyname(host)
        except ValueError as ex:
            logging.exception("message")
            raise KeySqlConnectException(ex)
        self._port = port
        self._server_address = (self._host, self._port)
        self._sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        if timeout:
            self._sock.settimeout(timeout)
        self._sock_connected = False
        self._connected = False
        self._msg_id = 0
        self._sending = False

    def connect_user(self, user, password, schema = None):
        reply = None
        try:
            init_msg = None
            if schema:
                init_msg = "connect --user " + user + " --password " + password + " --schema " + schema + "\n"
            else:
                init_msg = "connect " + user + " " + password + "\n"
            self._sock.sendall(bytes(init_msg,encoding="utf-8"))
            reply = self._sock.recv(128)
            pos = reply.find(b'\n')
            reply = reply[0:pos].decode("ascii")
        except ValueError as ex:   #   Exception as ex:
            raise KeySqlConnectException(ex)  # .message)

        if reply != "OK":
            #print(reply)
            raise KeySqlConnectException(reply)
        self._connected = True
        return True

    def __API_call(self,command: str,expected: str):
        reply = None
        try:
            while (self._sending):
                time.sleep(0.001)
            self._sending = True
            #logging.info("start_sending1:  " + str(datetime.now().time()))
            self._sock.sendall(bytes(command + "\n",encoding="utf-8"))
            #logging.info("finish_sending1: " + str(datetime.now().time()))
            self._sending = False
            if not expected:
                return True
            reply = self._sock.recv(128)
            pos = reply.find(b'\n')
            reply = reply[0:pos].decode("ascii")
        except ValueError as ex:   #   Exception as ex:
            raise KeySqlConnectException(ex)  # .message)

        if reply == expected:
            return True
        raise KeySqlConnectException(reply)  # .message)

    def disconnect_user(self):
        return self.__API_call("disconnect","OK")

    def abort_query(self):
        return self.__API_call("abort","")

    def shutdown(self):
        # TODO: check connections ---> ssl or not .... if ssl - False, for tcp only ---
        if True:
            return self.__API_call("shutdown", "")

    def ping(self):
        return self.__API_call("ping","PONG")

    def reconnect(self,schema : str):
        return self.__API_call("reconnect --schema " + schema,"OK")

    def change_password(self,username : str, old_password :str, new_password : str):
        return self.__API_call("change-password --user " + username + " --old-password " + old_password + " --new-password " + new_password, "OK")

    def _connect(self, user: str = None, password: str = None, schema: str = None):  # , nonblocking=True):
        """
        Connect with server: non-secure connect
        :param user: user name
        :param password: user password
        :return: None
        """

        if self._connected:
            raise KeySqlConnectException(KeySqlExceptionMessage.DISCONNECT_BEFORE_CONNECT)

        if not self._sock_connected:
            try:
                self._sock.connect(self._server_address)

                #self._sock.setblocking(0) # does not work, but doc says it should be
                #if nonblocking:
                #    self._sock.setblocking(0)

                self._sock_connected = True
            except ValueError as ex:  # Exception as ex:
                raise KeySqlConnectException(ex)  # .message)

        if user and password:
            return self.connect_user(user, password, schema)
        return True

    def ssl_connect(self, user: str = None, password: str = None, schema: str = None):
        """
        Connect with server: SSL connection with server
        :param user: user name
        :param password:  user password
        :return: None
        """
        if self._connected:
            raise KeySqlConnectException(KeySqlExceptionMessage.DISCONNECT_BEFORE_CONNECT)

        if not self._sock_connected:
            try:
                context = ssl.SSLContext(ssl.PROTOCOL_TLSv1_2)
                fd, tmp_path = tempfile.mkstemp()
                with open(tmp_path, 'w') as f:
                    for s in self.pem:
                        f.write(s)
                os.close(fd)
                context.load_verify_locations(tmp_path)
                self._sock = context.wrap_socket(self._sock, server_hostname = self._host)
                self._sock.connect(self._server_address)
                #self._sock.setblocking(0)
                #if nonblocking:
                #    self._sock.setblocking(0)
                #os.remove(tmp_path)
                self._sock_connected = True

            except ValueError as ex:
                raise KeySqlConnectException(ex)

        if user and password:
            return self.connect_user(user, password, schema)
        return True


    def send_resv(self, msg, format='keysql'):
        """
        Send a query to the server
        :param msg: str
        :return: None
        """

        def utf8len(s):
            return len(s.encode('utf-8'))

        if format not in ['keysql', 'json']:
            raise KeySqlException(-1, KeySqlExceptionMessage.FORMAT_KEYSQL_JSON)

        self._msg_id += 1
        header = MessageHeader(self._msg_id, format)
        message = "submit " + header.out_msg(utf8len(msg))
        message += msg

        while (self._sending):
            time.sleep(0.001)
        self._sending = True
        #logging.info("start_sending2:  " + str(datetime.now().time()))
        self._sock.sendall(message.encode())
        #logging.info("finish_sending2: " + str(datetime.now().time()))
        self._sending = False

        reply = ""
        reply_header = None
        header_str = ""
        page = b""

        def get_header(chunk,header_str):
            pos = chunk.find(b'\n')
            if pos > 0:
                header_str += chunk[0:pos].decode("ascii")
                reply_header = header.unpack(header_str)
                return chunk[pos + 1:], "", reply_header

            header_str += chunk.decode("ascii")
            return b"", header_str, None

        while True:
            chunk = self._sock.recv(8192)
            if reply_header is None:
                chunk, header_str, reply_header = get_header(chunk, header_str)
                #if reply_header:
                #   print("=============================", reply_header, len(page),  "=============================xxx")
                if reply_header and reply_header['exit_code'] != 0:
                    page = b""
                    reply = ""
            page += chunk
            if len(page) >= reply_header["message_length"]:
                reply += page[0:reply_header["message_length"]].decode("utf-8")
                if reply_header["page"] == 0:
                    break
                page, header_str, reply_header = get_header(page[reply_header["message_length"] + 1:],header_str)
                #if reply_header:
                #    print("=============================", reply_header, len(page),  "=============================yyy")

                if reply_header and reply_header["page"] == 0 and len(page) >= reply_header["message_length"]:
                    if reply_header['exit_code'] != 0:
                        reply = page.decode("utf-8")
                    else:
                        reply += page.decode("utf-8")
                    break;

        if reply_header['exit_code'] != 0:
            short_reply = self.cut_message(reply)
            logging.error('exit_code: ' + str(reply_header['exit_code']) + ', reply: ' + short_reply)
            reply_header["err_message"] = short_reply
            raise KeySqlException(reply_header['exit_code'], short_reply)

        return reply_header, reply

    # do not unpack reply, test only server process query, without unpacking reply
    # return total # of bytes
    def send_skip_resv(self, msg, format='keysql'):
        """
        Send a query to the server
        :param msg: str
        :return: None
        """

        def utf8len(s):
            return len(s.encode('utf-8'))

        if format.lower() not in ['keysql', 'json']:
            raise KeySqlException(-1, KeySqlExceptionMessage.FORMAT_KEYSQL_JSON)

        self._msg_id += 1
        header = MessageHeader(self._msg_id, format)
        message = "submit " + header.out_msg(utf8len(msg))
        message += msg

        while (self._sending):
            time.sleep(0.001)
        self._sending = True
        #logging.info("start_sending2:  " + str(datetime.now().time()))
        self._sock.sendall(message.encode())
        #logging.info("finish_sending2: " + str(datetime.now().time()))
        self._sending = False

        #reply = ""
        reply_headers = []
        reply_header = None
        header_str = ""
        page = b""

        def get_header(chunk,header_str):
            pos = chunk.find(b'\n')
            if pos > 0:
                header_str += chunk[0:pos].decode("ascii")
                reply_header = header.unpack(header_str)
                return chunk[pos + 1:], "", reply_header

            header_str += chunk.decode("ascii")
            return b"", header_str, None

        while True:
            chunk = self._sock.recv(8192)
            if reply_header is None:
                chunk, header_str, reply_header = get_header(chunk, header_str)
                #if reply_header:
                #   print("=============================", reply_header, len(page),  "=============================xxx")
                if reply_header and reply_header['exit_code'] != 0:
                    page = b""
                    #reply = ""
            page += chunk
            if len(page) >= reply_header["message_length"]:
                #reply += page[0:reply_header["message_length"]].decode("utf-8")
                if reply_header["page"] == 0:
                    break
                page, header_str, reply_header = get_header(page[reply_header["message_length"] + 1:],header_str)
                #if reply_header:
                #    print("=============================", reply_header, len(page),  "=============================yyy")

                if reply_header and reply_header["page"] == 0 and len(page) >= reply_header["message_length"]:
                    #if reply_header['exit_code'] != 0:
                    #    reply = page.decode("utf-8")
                    #else:
                    #    reply += page.decode("utf-8")

                    #break
                    reply_headers.append(reply_header)

                if reply_header and reply_header["query"] == 0:
                    break

        if reply_header['exit_code'] != 0:
            short_reply = "Error from server (when skip reply)"  # self.cut_message(reply)
            logging.error('exit_code: ' + str(reply_header['exit_code']) + ', reply: ' + short_reply)
            reply_header["err_message"] = short_reply
            raise KeySqlException(reply_header['exit_code'], short_reply)

        return reply_header  #  , reply


    def _disconnect(self):
        """
        Disconnect with server
        :return: None
        """
        time.sleep(0.5)     # 5 millisecond - to finish if something still running
        self._sock.close()
        self._connected = False


if __name__ == "__main__":
    pass

    #for testing
    """
    tracemalloc.start()
    my_local_host = 'my_ip_test'   #  change to real ip number 'a.b.c.d'
    my_local_host = 'localhost'

    #print('-------------------1')
    my_local_host = '12.3.4.5678'   # put correct host
    my_port = 1234                  # put correct  SSl port
    my_user = "user_name"           # put correct user name

    my_pass = "user_password"       # put correct password
    my_schema = 'user_schema'       # # put correct schema

    #print('-------------------2')
    # to run - uncomment and run with correct credentials
    connector = KeySqlConnector(my_local_host, my_port)
    connector.ssl_connect()

    connector.connect_user(my_user, "keysqldemo", "keysqluser".upper())

    connector.disconnect_user()

    #print('change-password: ', connector.change_password("x_x", "x_xx", "x_x"))
    #print('disconnect_user: ', connector.disconnect_user())
    print('ping: ', connector.ping())
    connector.connect_user("keysqluser", "keysqldemo", "keysqluser".upper())
    print('connected 2:', connector.is_connected())

    header, reply = '', ''
    #logging.basicConfig(level=logging.INFO)
    # '''
    try:
        header, reply = connector.send_resv('select * from world_bank_store')
        print('header no exception: ', header)
        print('reply no exception:', reply[0:1000], len(reply))
        header, reply = connector.send_resv('show catalogs')
        print('header no exception: ', header)
        print('reply no exception:', reply)
        header, reply = connector.send_resv('select * from billiard_store')
        print('header no exception: ', header)
        print('reply no exception:', reply)
        connector.reconnect("test".upper())
        header, reply = connector.send_resv('show catalogs')
        print('reply no exception:', reply)
        connector._disconnect()


    except KeySqlException as error:
        print('show non-existing catalog error code: {}, error message: {}'.format(error.error_code, error.error_text))
        print('header exception: ', header)
        print('reply exception:', reply)
    """
