#/**********************************************************************************
#* Copyright (C) 2017-2020 by KEYARK, INC. - All Rights Reserved.
#* All information contained herein is proprietary and confidential to KEYARK, INC.
#* Any use, reproduction, or disclosure without the written
#* permission of KEYARK, INC. is prohibited.
#* Written by Natalia Belyaeva <nbelyaeva@keyark.com>
#***********************************************************************************/

#from unit_tests_keysql_driver.test_keysql_mapper import *

from keysql_client.key_object import KeyObjectMultiple, KeyObjectComposition, KeyObjectBuilder
from keysql_client.keysql_catalog import KeySqlCatalog

class KeyObjectsReader_from_strings():
    dd = {}
    dd_unprocessed = {}
    processed_items = []

    def __init__(self):
        self.dd = {}
        self.dd_unprocessed = {}
        self.processed_items = []

    def process_object(self, sss):
        global processed_items
        global dd_unprocessed

        start = sss.find('{')
        end = sss.find('}')
        parent = sss[:start].strip()
        if parent in self.dd:
            return

        children = sss[start + 1: end].strip()
        if children.endswith('MULTIPLE'):
            child, _ = children.split(' ')
            if child in self.dd_unprocessed:
                self.process_object(self.dd_unprocessed[child])
                del self.dd_unprocessed[child]
            k_object = KeyObjectMultiple(parent, self.dd[child])
            self.dd[k_object.__name__] = k_object
            self.processed_items.append(k_object)
        else:
            children = sss[start+1:end].strip().split(',')
            children_classes = []
            for child in children:
                if child in self.dd_unprocessed:
                    self.process_object(self.dd_unprocessed[child])
                    del self.dd_unprocessed[child]
                children_classes.append(self.dd[child])
            k_object = KeyObjectComposition(parent, children_classes)
            self.dd[k_object.__name__] = k_object
            self.processed_items.append(k_object)

    def get_keyobjects_from_strings(self, reply11):
        primitive_items = [item for item in reply11 if '{' not in item]
        complex_items = [item for item in reply11 if '{' in item]

        for item in primitive_items :
            k_object = KeyObjectBuilder().create_keysql_object_from_string(item)
            #self.dd[k_object.__name__] = k_object
            self.dd[k_object.__name__] = k_object
            self.processed_items.append(k_object)

        for sss in complex_items:
            start = sss.find('{')
            end = sss.find('}')
            parent = sss[:start].strip()
            self.dd_unprocessed[parent] = sss

        for item in complex_items:
            self.process_object(item)

        return self.processed_items

    """
    # Open a file: file
    file = open('my_text_file',mode='r')

    # read all lines at once
    all_of_it = file.read()

    # close the file
    file.close()
    """

class KeyObjectsReader_from_file():
    @staticmethod
    def read_query_list_from_file(input_file):
        """
        every line - one query
        :param input_file:
        :return: list of queries
        """
        query_list = []
        with open(input_file, 'r',  encoding="utf-8") as ff:
            query_list = [line.strip() for line in ff]
            query_list = [qq for qq in query_list if len(qq) > 0]
        return query_list

    @staticmethod
    def read_whole_query_list_from_file(input_file):
        """
        read the whole file as one query - for insert mostly
        :param input_file:
        :return: quey
        """
        with open(input_file, 'r',  encoding="utf-8") as ff:
            query = ff.read().strip()
            return query


    @staticmethod
    def parse_catalog_keyobjects_instanses(input_file):
        catalog = ""
        read_keyobjects = []
        read_instanses = []
        with open(input_file, 'r',  encoding="utf-8") as ff:
            catalog = ff.readline().strip().split()[1]

            done = False
            while not done:
                current_ko = ff.readline().strip()
                # blank line - separator between kyeobjects and instances
                #if len(current_ko) < 3:
                #    print(len(current_ko), current_ko)
                if len(current_ko) < 3:
                    done = True
                    break
                # remove first {
                if len(read_keyobjects) == 0:
                    current_ko = current_ko[1:]
                read_keyobjects.append(current_ko[:-1]) # whether , or last }
            #print(read_keyobjects)

            ko_reader = KeyObjectsReader_from_strings()
            key_objects = ko_reader.get_keyobjects_from_strings(read_keyobjects)

            for line in ff:
                line = line.strip()
                if len(line) == 0:
                    continue
                else:
                    if line[-1] == ',':
                        line = line[:-1]
                    read_instanses.append(line)
            #print(read_instanses)

            instanses = []
            catalog = KeySqlCatalog(catalog)

            return catalog, key_objects, read_instanses

if __name__ == "__main__":

    reply11 = ['AGGR_RESULT {CNT,MAX_STR}', 'BALL {COLOR,NUM}', 'BALLPACK {BALLSET,BRAND,NAME,UPC}', 'BALLS {BALL MULTIPLE}',
     'BALLSET {BALLS,DIAM_MM,QTY,STYLE}', 'BRAND CHAR', 'BUILDING {HOUSE_NUM,NAME,STREET,TENANTS}',
     'BUILDINGS {BUILDING MULTIPLE}', 'CITY CHAR', 'CNT NUMBER', 'CNT1 NUMBER', 'CNT2 NUMBER', 'COLOR CHAR',
     'DIAM_MM NUMBER', 'FIRST_NAME CHAR', 'FOOTAGE NUMBER', 'FULL_NAME {FIRST_NAME,LAST_NAME}',
     'FULL_NAMES {FULL_NAME MULTIPLE}', 'HOUSE_NUM NUMBER', 'LANDLORD {BUILDINGS,MANAGER,NAME}',
     'LANDLORD_P {BUILDINGS,MANAGER_P,NAME}', 'LAST_NAME CHAR', 'MANAGER {FIRST_NAME,LAST_NAME,OFFICE_ADDRESS,PHONE}',
     'MANAGER_P {OFFICE_ADDRESS,PERSON,PHONE}', 'MAX_COLOR CHAR', 'MAX_NUM NUMBER', 'MAX_STR CHAR', 'NAME CHAR',
     'NUM NUMBER', 'OFFICE_ADDRESS {CITY,HOUSE_NUM,ROOM_NUM,STREET,ZIPCODE}', 'PEOPLE {PERSON MULTIPLE}',
     'PERSON {FIRST_NAME,LAST_NAME,YOB}', 'PHONE NUMBER', 'QTY NUMBER', 'RATING NUMBER',
     'ROOM {FOOTAGE,RATING,ROOM_NUM}', 'ROOMS {ROOM MULTIPLE}', 'ROOM_NUM NUMBER', 'STREET CHAR', 'STYLE CHAR',
     'TENANT {NAME,PEOPLE,ROOMS}', 'TENANTS {TENANT MULTIPLE}', 'TWO_LANDLORDS {LANDLORD,LANDLORD_P}', 'UPC NUMBER',
     'YOB NUMBER', 'ZIPCODE NUMBER']

    extract_key_objects_engine = KeyObjectsReader_from_strings()
    processed_items = extract_key_objects_engine.get_keyobjects_from_strings(reply11)



