#/**********************************************************************************
#* Copyright (C) 2017-2020 by KEYARK, INC. - All Rights Reserved.
#* All information contained herein is proprietary and confidential to KEYARK, INC.
#* Any use, reproduction, or disclosure without the written
#* permission of KEYARK, INC. is prohibited.
#* Written by Natalia Belyaeva <nbelyaeva@keyark.com>
#**********************************************************************************/

from keysql_client.keysql_instance import *

class KeySqlParser(object):
    def __init__(self):    # input_list_dict_instance):
        pass

    @staticmethod
    def parse_name(_input, pos, total_len):
        for i in range(pos, total_len):
            if _input[i] == ":":
                return i
        return -1

    @staticmethod
    def parse_char_value_old(_input, pos, total_len):
        for i in range(pos + 1, total_len):
            if _input[i] == "'":
                return i
        return -1

    def parse_char_value(_input, pos, total_len):
        #for i in range(pos + 1, total_len):
        i = pos + 1
        while i < total_len:
            if _input[i] == "'":
                if _input[i + 1] == "'":
                    i += 1
                else:
                    return i
            i += 1
        return -1

    @staticmethod
    def parse_value(_input, pos, total_len):
        for i in range(pos, total_len):
            if _input[i] == "," or _input[i] == "}":
                return i
        return -1

    def next_instance(_input, pos):
        for i in range(pos,len(_input)):
            if _input[i] == "{":
                return i
        return -1

    @staticmethod
    def instance_parser(_input, pos):
        total_len = len(_input)
        start = pos
        pos = KeySqlParser.parse_name(_input,pos,total_len)
        if pos == -1:
            return None, pos
        name = _input[start : pos]
        pos += 1 # skip :

        if _input[pos] == "'":
            start = pos
            pos = KeySqlParser.parse_char_value(_input,pos,total_len)
            if pos == -1:
                return None, pos
            value = _input[start + 1 : pos]
            value = value.replace("''", "'")
            pos += 1
            return KeySql_Instance(name, value), pos

        if _input[pos] == "{":
            if name == 'RESULT':
                value = []
                is_map = False
            else:
                value = {}
                is_map = True
            while pos < total_len:
                child, pos = KeySqlParser.instance_parser(_input, pos + 1)
                if child is None:
                    return None, pos
                if is_map:
                    check = value.get(child._name)
                    if check:
                        value = [check,child]
                        is_map = False
                    else:
                        value[child._name] = child
                else:
                    value.append(child)
                if _input[pos] == ",":
                    continue
                if _input[pos] != "}":
                    return None, pos
                pos += 1
                break
            return KeySql_Instance(name, value), pos

        start = pos
        pos = KeySqlParser.parse_value(_input,pos,total_len)
        if pos == -1:
            return None, pos
        val = _input[start : pos]
        if val == "NULL":
            val = None
        else:
            val = float(val)
            if val % 1 == 0:
                val = int(val)

        return KeySql_Instance(name, val), pos

    @staticmethod
    def parse_select_reply_all(_input):

        print("---{}===".format(_input), '_________________________________________________ parse_select_reply xxx ')

        if len(_input) == 0:
            return []

        if _input[0] == '[' and _input[-1] == ']':
            _input = _input[1:-1]

        if len(_input) > 0:
            if _input[0] != "{":   # always start with {
                raise KeySqlException(KeySqlExceptionMessage.SELECT_REPLY_SHOULD_START_WITH_CURLY_BRACKET)

        instances = []
        pos = 0
        while pos < len(_input):
            if pos >= 0 and _input[pos] == "{":
                pos += 1
            else:
                break
            instance, pos = KeySqlParser.instance_parser(_input, pos)
            if instance:
                instances.append(instance)
                if pos >= len(_input):
                    return instances
                pos = KeySqlParser.next_instance(_input, pos)
            else:
                return None
        return instances


    @staticmethod
    def parse_select_reply(_input):

        print("---{}===".format(_input), '_________________________________________________ parse_select_reply xxx ')

        if len(_input) == 0:
            return []

        if _input[0] == '[' and _input[-1] == ']':
            _input = _input[1:-1]

        if len(_input) > 0:
            if _input[0] != "{":   # always start with {
                raise KeySqlException(KeySqlExceptionMessage.SELECT_REPLY_SHOULD_START_WITH_CURLY_BRACKET)

        instances = []
        pos = 0
        while pos < len(_input):
            if pos >= 0 and _input[pos] == "{":
                pos += 1
            else:
                break
            instance, pos = KeySqlParser.instance_parser(_input, pos)
            if instance:
                instances.append(instance)
                if pos >= len(_input):
                    return instances
                pos = KeySqlParser.next_instance(_input, pos)
            else:
                return None
        return instances

