#/**********************************************************************************
#* Copyright (C) 2017-2020 by KEYARK, INC. - All Rights Reserved.
#* All information contained herein is proprietary and confidential to KEYARK, INC.
#* Any use, reproduction, or disclosure without the written
#* permission of KEYARK, INC. is prohibited.
#* Written by Natalia Belyaeva <nbelyaeva@keyark.com>
#***********************************************************************************/

import sys
import inspect

#from keysql_client import *
import keysql_client
from keysql_client.keysql_instance import *
from keysql_client.keysql_catalog import *

from keysql_client.key_object import KeyObjectComposition, KeyObjectMultiple

ALL_KEY_SQL_CLASSES_NAMES = ['AlterKeyObjectStatement', 'CreateKeyObjectStatement', 'DeleteStatement',
    'DropKeyObjectStatement', 'Enum', 'InsertIntoStoreInstance', 'KEYOBJECT_TYPE', 'KeyObject', 'KeyObjectBuilder',
    'KeyObjectChar', 'KeyObjectComposition', 'KeyObjectDate', 'KeyObjectMultiple', 'KeyObjectNumber',
    'KeySqlAlterCatalogStatement', 'KeySqlAlterStoreNameStatement', 'KeySqlCatalog', 'KeySqlConnection',
    'KeySqlCreateCatalogStatement', 'KeySqlCreateStoreStatement', 'KeySqlDropCatalogStatement',
    'KeySqlDropStoreStatement', 'KeySqlException', 'KeySqlExceptionMessage', 'KeySqlParserQueryParameterized',
    'KeySqlShowCatalogStatement', 'KeySqlShowCatalogsStatement', 'KeySqlShowStoreStatement', 'KeySqlStatement',
    'KeySqlStore', 'KeySql_Instance', 'SelectStatement', 'ShowKeyObjectStatement', 'StatementObj',
    'StatementType', 'TestCase', 'TestKeySqlBase', 'TestKeysql_string_query_parameterized', 'UpdateStatement'
]

from keysql_client.keysql_store import KeySqlStore

ALL_KEY_SQL_CLASSES = [
    keysql_client.keysql_store.KeySqlStore,
    keysql_client.keysql_catalog.KeySqlCatalog,
    keysql_client.key_object_statement.CreateKeyObjectStatement,
    """
    keysql_client.key_object_statement.AlterKeyObjectStatement,
    keysql_client.keysql_delete.DeleteStatement,
    keysql_client.key_object_statement.DropKeyObjectStatement,
    keysql_client.keysql_insert.InsertIntoStoreInstance,
    #KEYOBJECT_TYPE,
    keysql_client.key_object.KeyObject,
    keysql_client.key_object.KeyObjectBuilder,
    keysql_client.key_object.KeyObjectChar,
    keysql_client.key_object.KeyObjectComposition,
    keysql_client.key_object.KeyObjectDate,
    keysql_client.key_object.KeyObjectMultiple,
    keysql_client.key_object.KeyObjectNumber,
    keysql_client.keysql_catalog_statement.KeySqlAlterCatalogStatement,
    keysql_client.keysql_store_statement.KeySqlAlterStoreNameStatement,
    keysql_client.keysql_catalog.KeySqlCatalog,
    #keysql_client.keysql_connection.KeySqlConnection,
    keysql_client.keysql_catalog_statement.KeySqlCreateCatalogStatement,
    keysql_client.keysql_store_statement.KeySqlCreateStoreStatement,
    keysql_client.keysql_catalog_statement.KeySqlDropCatalogStatement,
    keysql_client.keysql_store_statement.KeySqlDropStoreStatement,
    keysql_client.keysql_exception.KeySqlException,
    keysql_client.keysql_exception.KeySqlExceptionMessage,
    #keysql_client.keysql_parser_query_parameterized.KeySqlParserQueryParameterized,
    keysql_client.keysql_catalog_statement.KeySqlShowCatalogStatement,
    keysql_client.keysql_catalog_statement.KeySqlShowCatalogsStatement,
    keysql_client.keysql_store_statement.KeySqlShowStoreStatement,
    keysql_client.keysql_statement.KeySqlStatement,
    keysql_client.keysql_store.KeySqlStore,
    keysql_client.keysql_instance.KeySql_Instance,
    keysql_client.keysql_select.SelectStatement,
    keysql_client.key_object_statement.ShowKeyObjectStatement,
    #StatementObj'>, <enum 'StatementType'>,
    keysql_client.keysql_update.UpdateStatement
    """
]


"""
how to get classes:
clsmembers = inspect.getmembers(sys.modules[__name__], inspect.isclass)
key_sql_classes = [nm for nm, _cl in clsmembers]
"""

class KeySqlParserQueryParameterized(object):

    @staticmethod
    def process_one_elem(arg1):
        if arg1 is None:
            return "NULL"
        elif type(arg1) is int:
            return arg1
        #elif type(arg1) is dict:
        #    processed_args.append(arg1)
        elif type(arg1) is str:
            return arg1
        elif type(arg1) in [KeyObject, KeyObjectChar, KeyObjectDate, KeyObjectComposition, KeyObjectMultiple]:  # is KeyObject:
            tmp = arg1.get_name()  #  arg1.serialize()
            return tmp

        tmp_type = type(arg1)
        res = tmp_type in ALL_KEY_SQL_CLASSES
        res1 = tmp_type is keysql_client.keysql_store.KeySqlStore

        if  type(arg1) in ALL_KEY_SQL_CLASSES: # (type(arg1) is not dict) # (arg1.__name__ in ALL_KEY_SQL_CLASSES):       #   type(arg1) in ALL_KEY_SQL_CLASSES:
            return arg1.get_name()
        else:
            return KeySql_Instance.instances_to_string(arg1)


    @staticmethod
    def process_all_args(*args):

        list_args = []
        for arg1 in args:
            if type(arg1) is list:
                list_args.append(",".join(map(KeySqlParserQueryParameterized.process_one_elem, arg1)))

            else:
                #list_args.append(KeySql_Instance.instances_to_string(arg1))   # does not work
                #list_args.append(KeySqlParserQueryParameterized.process_one_elem(arg1))
                #"""
                tmp2 = KeySqlParserQueryParameterized.process_one_elem(arg1)
                list_args.append(KeySqlParserQueryParameterized.process_one_elem(arg1))
                #"""

            """    
            elif type(arg1) is dict:
                tmp_args = []
                for key in arg1:
                    tmp_args.append("{}:'{}'".format(key, KeySqlParserQueryParameterized.process_one_elem(arg1[key])))
                list_args.append(",".join(tmp_args))
            else:
                list_args.append(KeySqlParserQueryParameterized.process_one_elem(arg1))
            """
        return list_args


    @staticmethod
    def process_input_query(statement: str, *args) -> str:
        """Parametrized string query
        :param statement (str): string patter  of type     "##### ##### {} ##### {}" where {} is a placeholder for parameters
        example1:  process_input_query("create store {} in catalog {}", "store_name", KeySqlCatalog("catalog name")
        example2:  process_input_query("create catalogs {}",  ["cat1", "cat2"] )    - one parameter as list
        example3:  process_input_query("create catalogs {}, {}", KeySqlCatalog("cat1"), KeySqlCatalog("cat2")) - two
        parametes as classes
        :param args(str, int, KeySql Class, or dict, list of them): variable number of arguments ([0..Any])
        :return (str): string query
        """

        if statement.count("{}") != len(args):
            raise KeySqlException(-1, KeySqlExceptionMessage.NUMBER_PATTERNS_EQUALS_NUMBER_ARGUMENTS)

        list_args = KeySqlParserQueryParameterized.process_all_args(*args)

        #return statement.format(*list_args) # will not procees string like {{} }

        tmp = statement.split("{}")

        for i in range(len(list_args)):
            if type(args[i]) == str:
                tmp.insert(1 + 2 * i, "'" + list_args[i] + "'")
            else:
                tmp.insert(1+ 2*i, str(list_args[i]))

        final = "".join(tmp)

        return final.strip()

if __name__ == '__main__':
    pass
