#/**********************************************************************************
#* Copyright (C) 2017-2020 by KEYARK, INC. - All Rights Reserved.
#* All information contained herein is proprietary and confidential to KEYARK, INC.
#* Any use, reproduction, or disclosure without the written
#* permission of KEYARK, INC. is prohibited.
#* Written by Natalia Belyaeva <nbelyaeva@keyark.com>
#***********************************************************************************/
import logging
from typing import List
from keysql_client.keysql_statement import KeySqlStatement
from keysql_client.keysql_instance import KeySql_Instance
from keysql_client.keysql_exception import KeySqlException, KeySqlExceptionMessage

# TODO: not implemented in server code Not callable yet
class KeySqlVersionStatement(KeySqlStatement):
    """Get version of server and check that python driver version corresponds the server version
    Server version is in the Describe  String query equivalent:
    'describe ...'
    """

    def __init__(self) -> None:
        """
        :param no
        """
        #_describe_str = describe_str
        pass

    def get_server_version(self):
        pass

    def create_request(self) -> str:
        return "show version"


    def process_reply(self, reply: str, header=None) -> None:
        # nothing to process - input is ''

        if not reply or len(reply) == 0:
            return []

        # reply: like  [{"STORE_NAME":"DUMMY_STORE1"},{"STORE_NAME":"DUMMY_STORE2"}]
        if header and header['format'] == 'json':
            if type(reply) is list:
                #return [rec['version_info'].instance_value()['version_num'].instance_value() for rec in reply]
                return [rec['version_info']['version_num'].instance_value() for rec in reply]

        if header and header['format'] == 'keysql':
            if type(reply) is list:
                #print(reply, '--------------------------------------------err check')
                #print(reply[0], '--------------------------------------------err check')
                #print(reply[0].instance_name(), '--------------------------------------------err check')
                if reply[0].instance_name() != 'version_info':
                    raise KeySqlException(KeySqlExceptionMessage.INSTANCE_KEY_VERSION_INFO)
                #return [rec.instance_value().instance_value() for rec in reply]
                return [rec['version_info']['version_num'].instance_value() for rec in reply]

        """
        # always len(reply = 1;   [{key:val}]
        res  =  []
        #print(reply, '========================================================================= xxx')
        #print(type(reply), type(reply[0]), "=================")
        ##print(reply[0], '================================process_reply')
        if type(reply[0]) is KeySql_Instance:
            val = reply[0].instance_value().strip().split('\n')
            #print(val, "=================================================xxx1 instance")
        else:
            for key in reply[0].keys():
                val = reply[0][key].strip().split('\n')
                #print(val, "=================================================xxx1 json")
        #print(val, "=================================================xxx1")
        res = [ss for ss in val if len(ss) > 0 and not ss.startswith('--')]
        return res
        """

if __name__ == "__main__":
    version  = KeySqlVersionStatement()
    reply = version.get_server_version()
