from threading import Thread, Lock
from time import sleep
import logging
from keysql_client.keysql_connection import KeySqlConnection
from keysql_client.keysql_store import KeySqlStore
from keysql_client.keysql_db_reader_from_file import KeyObjectsReader_from_file
from keysql_client.keysql_catalog_statement import  KeySqlShowCatalogsStatement, KeySqlDropCatalogStatement

class KeySqlWorker(Thread):
    __lock = Lock()

    #def __init__(self, db, query, result_queue):
    def __init__(self, result_queue, my_host, my_port, my_name, my_password, ssl):

        Thread.__init__(self)
        self.queries = []
        self.result_queue = result_queue
        self.connection = None

        try:
            self.connection = KeySqlConnection(my_host, my_port, my_name, my_password, ssl=ssl)
        except Exception as e:
            logging.error("Unable to connect with server: {}".format(str(e)))

        connected = self.connection is not None and self.connection.is_connected()
        if not connected:
            logging.error("Not connected with server")

    # destructor
    def __del__(self):
        if self.connection is not None and self.connection.is_connected():
            self.connection.disconnect()

    def execute(self, query):
        result = None
        logging.info("Execute query:")
        logging.info(query)
        try:
            result = self.connection.execute(query, keep_str_res=True)
            logging.info(self.connection.get_last_str_result())

        except Exception as e:
            logging.error("Unable to execute querry: {}".format(str(e)))
        self.result_queue.append(result)

@staticmethod
def clean_catalog_for_connection(connection, catalog_name) -> None:
    # TODO - remove when delete catalog and store is fixed
    # return
    ########################################################
    #"""
    show_catalogs_statement = KeySqlShowCatalogsStatement()
    all_catalog_names = connection.execute(show_catalogs_statement)

    catalog_name = catalog_name.upper()
    if catalog_name in all_catalog_names:
        stores = connection.execute('show stores for catalog {}'.format(catalog_name))
        #print("stores: ", stores)
        # if there are stores, remove them
        for _store in stores:
            reply = connection.execute('drop store {}'.format(_store))

        connection.execute(KeySqlDropCatalogStatement(catalog_name))


def create_billiard_store(test_host, test_port, test_user_name, my_passport, ssl):

    my_connection = KeySqlConnection(test_host, test_port, test_user_name, my_passport, ssl=ssl)

    catalog_name = 'Universe'
    store_name = 'billiard_store'
    store = KeySqlStore('billiard_store')
    input_file = 'billiard_store.sql'

    # create catalog from scratch
    clean_catalog_for_connection(my_connection, catalog_name)

    #catalog, key_objects, instanses = self.parse_catalog_keyobjects_instanses(input_file)

    #ko_reader = KeyObjectsReader_from_file()
    catalog, key_objects, instanses = KeyObjectsReader_from_file.parse_catalog_keyobjects_instanses(input_file)

    my_connection.execute(catalog.create(), keep_str_res=True)
    my_connection.execute(catalog.create_store(store), keep_str_res=True)

    all_ko = [ catalog.create_object(ko) for ko in key_objects]
    my_connection.execute(all_ko)

    reply = my_connection.execute("show catalogs")
    assert catalog_name.upper() in reply
    #print(reply)

    reply = my_connection.execute("show catalog UNIVERSE", keep_str_res=True)
    #print(reply)
    #print(my_connection.get_last_str_result())
    assert len(reply) == 46

    reply = my_connection.execute("show stores for catalog UNIVERSE")
    #print(reply)
    assert reply == ['BILLIARD_STORE']

    cnt_inst = 0
    for inst in instanses:
        #print('cnt_inst: ', cnt_inst)
        cnt_inst += 1
        query = "INSERT INTO {} INSTANCES {}".format(store_name, (inst))
        #print()
        #print(query)

        count = my_connection.execute(query, keep_str_res=True)
        #print()
        #print(my_connection.get_last_str_result())
        #print()

    assert cnt_inst == 4
    my_connection.disconnect()



if __name__ == "__main__":
    pass
    #to test
    """
    my_port = 6667           # test_keysql_connection_params['test_port']
    my_host = '1.2.3.456'    # test_keysql_connection_params['test_local_host'] - to change to correct ip address
    my_name = 'test3'        # test_keysql_connection_params['test_user_name']
    my_password = 'test'     # test_keysql_connection_params['test_password']
    ssl = True               # test_keysql_connection_params['ssl']

    create_billiard_store(my_host, my_port, my_name, my_password, ssl)

    delay = 1
    result_queue = []
    worker1 = KeySqlWorker( result_queue, my_host, my_port, my_name, my_password, ssl)
    worker2 = KeySqlWorker( result_queue, my_host, my_port, my_name, my_password, ssl)
    worker1.start()
    worker1.execute("")
    worker2.start()

    # Wait for the job to be done
    while len(result_queue) < 2:
        sleep(delay)
    job_done = True
    worker1.join()
    worker2.join()

    print(worker1.result_queue)
    print(worker2.result_queue)
    """